
#include <gnome.h>
#include <libgnomeui/gnome-window-icon.h>
#include <stdlib.h>
#include <time.h>
#include "gmonsters.h"
#include "filedirs.h"
#include "config.h"

static void gm_battle_random_select_cb(GtkWidget *, GtkWidget *);
static void gm_battle_random_select_area_cb(GnomeDialog *, guint, GMAreas *);
static void gm_battle_random_cb(GMAreas);
static void gm_battle_trainer_select_cb(GtkWidget *, GtkWidget *);
static void gm_battle_trainer_cb();
static void gm_battle_nethost_select_cb(GtkWidget *, GtkWidget *);
static void gm_battle_nethost_cb();
static void gm_battle_network_select_cb(GtkWidget *, GtkWidget *);
static void gm_battle_network_cb(gchar *);
static void gm_battle_network_ip(gchar *, gpointer data);
static gboolean close_out = FALSE;
static void gm_close() { close_out = TRUE; }

void gmbattle_main() {
	GtkWidget *window;
	GtkWidget *vbox;
	GtkWidget *random;
	GtkWidget *trainer;
	GtkWidget *nethost;
	GtkWidget *network;
	GtkWidget *quit;
//	gnome_init(PACKAGE, VERSION, argc, argv);
//	gm_init();
	close_out = FALSE;
	window = gnome_app_new("Gmonsters", "GMonsters: Battle");
	gnome_window_icon_set_default_from_file(gm_pixmap_file("gnome-gmbattle.png"));
	vbox = gtk_vbox_new(TRUE, 0);
	random = gtk_button_new_with_label("VS. Random monster");
	trainer = gtk_button_new_with_label("VS. Random trainer");
	nethost = gtk_button_new_with_label("Network Host");
	network = gtk_button_new_with_label("Network Challenger");
	quit = gtk_button_new_with_label("Quit Battle");
	gtk_box_pack_start(GTK_BOX(vbox), random, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), trainer, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), nethost, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), network, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), quit, FALSE, FALSE, 0);
	gnome_app_set_contents(GNOME_APP(window), vbox);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(&gm_close), NULL);
	gtk_signal_connect(GTK_OBJECT(random), "clicked", GTK_SIGNAL_FUNC(&gm_battle_random_select_cb), window);
	gtk_signal_connect(GTK_OBJECT(trainer), "clicked", GTK_SIGNAL_FUNC(&gm_battle_trainer_select_cb), window);
	gtk_signal_connect(GTK_OBJECT(nethost), "clicked", GTK_SIGNAL_FUNC(&gm_battle_nethost_select_cb), window);
	gtk_signal_connect(GTK_OBJECT(network), "clicked", GTK_SIGNAL_FUNC(&gm_battle_network_select_cb), window);
	gtk_signal_connect(GTK_OBJECT(quit), "clicked", GTK_SIGNAL_FUNC(&gm_close), NULL);
#ifndef TESTNET
	gtk_widget_set_sensitive(GTK_WIDGET(nethost), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(network), FALSE);
#endif
	gtk_widget_show_all(window);
	while(close_out == FALSE)
		while(gtk_events_pending())
			gtk_main_iteration();
	gtk_widget_destroy(window);
	gnome_config_sync();
	return 0;
}

static void
gm_battle_random_select_cb(GtkWidget * widget, GtkWidget * window)
{
	GtkWidget *dia = gnome_dialog_new("GMonsters: Select Area", "Plains", "Forest",
									  "Caves", "Mountains", "Oceans", "Rivers",
									  NULL);
	GMAreas area = 0;
	gtk_widget_hide(window);
	gtk_signal_connect(GTK_OBJECT(dia), "clicked", GTK_SIGNAL_FUNC(&gm_battle_random_select_area_cb), &area);
	gnome_dialog_set_close(GNOME_DIALOG(dia), TRUE);
	gnome_dialog_run(GNOME_DIALOG(dia));
	gm_battle_random_cb(area);
	gtk_widget_show_all(window);
}

static void
gm_battle_random_select_area_cb(GnomeDialog * dia, guint num, GMAreas * area)
{
	switch (num) {
	case 0:
		*area = GM_PLAINS;
		break;
	case 1:
		*area = GM_FOREST;
		break;
	case 2:
		*area = GM_CAVES;
		break;
	case 3:
		*area = GM_MOUNTAINS;
		break;
	case 4:
		*area = GM_OCEANS;
		break;
	case 5:
		*area = GM_RIVERS;
		break;
	default:
		*area = 0;
	}
}

static void
gm_battle_random_cb(GMAreas area)
{
	GMSpecies *spec;
	GMMonster *mon, *pmon = NULL;
	GMTrainer *monlist = gm_trainer_new();
	guint rnd, a, lvl = 0, div = 0;
	gulong id;
	gboolean rv;
	gchar *strng;
	srand((int)time(NULL));
	free(monlist->name);
	monlist->name = "Wild";
	monlist->skill.normal = 20;
	monlist->skill.stone = 20;
	monlist->skill.flying = 20;
	monlist->skill.electric = 20;
	monlist->skill.water = 20;
	monlist->skill.fire = 20;
	monlist->skill.poison = 20;
	monlist->skill.insect = 20;
	monlist->skill.plant = 20;
	monlist->skill.fighting = 20;
	monlist->skill.psychic = 20;
	monlist->skill.fungus = 20;
	monlist->skill.nocturnal = 20;
	monlist->skill.cybernetic = 20;
	monlist->skill.ice = 20;
	for (a = 0; a < gm_trainer->nom; a++) {
		if (gm_trainer->monsters[a]->active == TRUE) {
			lvl += (pmon = gm_monster_open(gm_trainer->monsters[a]->id))->lvl;
			div++;
		}
		gm_monster_close(gm_trainer->monsters[a]->id);
	}
	if (pmon != NULL) {
		lvl /= div;
		do {
			rnd = rand() % gm_species_list_length();
			spec = (GMSpecies *) gm_species_get_list() + rnd;
		}
		while (!(spec->areas & area) && area != 0);
		strng = g_strdup_printf("/GMonsters/Species Seen/%s", spec->name);
		gnome_config_set_bool(strng, TRUE);
		free(strng);
		strng = g_strdup_printf("Wild %s", spec->name);
		mon = gm_monster_new(spec, strng, (rand() % 2 == 0 ? GM_MALE : GM_FEMALE), lvl + (rand() % 8) - 4);
		free(strng);
		monlist->nom = 1;
		monlist->monsters = malloc(sizeof(GMMonsterList *));
		monlist->monsters[0] = malloc(sizeof(GMMonsterList));
		monlist->monsters[0]->active = TRUE;
		monlist->monsters[0]->name = g_strdup(mon->name);
		monlist->monsters[0]->id = mon->id;
		for (a = 0; a < mon->noa; a++)
			mon->abilities[a]->active = FALSE;
		for (a = 0; a < 5; a++)
			mon->abilities[rand() % mon->noa]->active = TRUE;
		mon->obe = 100;
		id = mon->id;
		monlist->monselect = &gm_trainer_ai_select;
		monlist->itemselect = &gm_trainer_item_ai_select;
		rv = gm_battle_start(gm_trainer, &gm_battle_player_cb, monlist, &gm_battle_ai_cb, TRUE, GM_SINGLE_PLAYER, TRUE);
		if (rv == GM_CAPTURE) {
			mon = gm_monster_open(mon->id);
			free(mon->name);
			mon->name = g_strdup(spec->name);
			mon->obe = 2 + (rand() % 9);
			gm_trainer_add_monster(gm_trainer, mon);
			gm_monster_close(mon->id);
		} else
			gm_monster_destroy(mon->id);
		free(monlist);
	}
}

static void
gm_battle_trainer_select_cb(GtkWidget * widget, GtkWidget * window)
{
	gtk_widget_hide(window);
	gm_battle_trainer_cb();
	gtk_widget_show_all(window);
}

static void
gm_battle_trainer_cb()
{
	GMSpecies *spec;
	GMMonster *mon, *pmon = NULL;
	GMTrainer *monlist = gm_trainer_new();
	guint rnd, rnd2, rnd3, a, b, lvl = 0, div = 0;
	gchar *strng;
	srand((int)time(NULL));
	monlist->skill.normal += rand() % 80;
	monlist->skill.stone += rand() % 80;
	monlist->skill.flying += rand() % 80;
	monlist->skill.electric += rand() % 80;
	monlist->skill.water += rand() % 80;
	monlist->skill.fire += rand() % 80;
	monlist->skill.poison += rand() % 80;
	monlist->skill.insect += rand() % 80;
	monlist->skill.plant += rand() % 80;
	monlist->skill.fighting += rand() % 80;
	monlist->skill.psychic += rand() % 80;
	monlist->skill.fungus += rand() % 80;
	monlist->skill.nocturnal += rand() % 80;
	monlist->skill.cybernetic += rand() % 80;
	monlist->skill.ice += rand() % 80;
	for (a = 0; a < gm_trainer->nom; a++) {
		if (gm_trainer->monsters[a]->active == TRUE) {
			lvl += (pmon = gm_monster_open(gm_trainer->monsters[a]->id))->lvl;
			div++;
		}
		gm_monster_close(gm_trainer->monsters[a]->id);
	}
	if (pmon != NULL) {
		gint forkrv;
		lvl /= div;
		rnd = (rand() % 5) + 1;
		monlist->nom = rnd;
		monlist->monsters = malloc(sizeof(GMMonsterList *) * rnd);
		for (a = 0; a < rnd; a++) {
			rnd2 = rand() % gm_species_list_length();
			spec = (GMSpecies *) gm_species_get_list() + rnd2;
			strng = g_strdup_printf("/GMonsters/Species Seen/%s", spec->name);
			gnome_config_set_bool(strng, TRUE);
			free(strng);
			rnd3 = lvl + ((rand() % 14) - 7);
			strng = g_strdup_printf("Trainer's %s", spec->name);
			mon = gm_monster_new(spec, strng, (rand() % 2 == 0 ? GM_MALE : GM_FEMALE), (rnd3 < 100 ? rnd3 : 99));
			free(strng);
			for (b = 0; b < mon->noa; b++)
				mon->abilities[b]->active = FALSE;
			for (b = 0; b < 5; b++)
				mon->abilities[rand() % mon->noa]->active = TRUE;
			mon->obe = 5 + (rand() % 96);
			monlist->monsters[a] = malloc(sizeof(GMMonsterList));
			monlist->monsters[a]->active = TRUE;
			monlist->monsters[a]->name = g_strdup(mon->name);
			monlist->monsters[a]->id = mon->id;
		}
		gm_monster_close(pmon->id);
		monlist->monselect = &gm_trainer_ai_select;
		monlist->itemselect = &gm_trainer_item_ai_select;
#ifdef TESTNET
		forkrv = fork();
		if (forkrv == -1)
			fprintf(stderr, __FILE__ ": %d: fork() to start server failed.\n", __LINE__);
		else if (forkrv == 0) {
			GMTrainer *trn = NULL;
			gm_server_start();
			fprintf(stderr, "Proccess %d: " __FILE__ ": %d: fakenet server initialized.\n", getpid(), __LINE__);
			trn = gm_trainer;
			gm_trainer = NULL;
			trn->monselect = &gm_trainer_net_select;
			trn->itemselect = &gm_trainer_item_net_select;
			gm_battle_start(trn, &gm_battle_net_cb, monlist, &gm_battle_ai_cb, FALSE, GM_MULTIPLAYER_HOST, FALSE);
			fprintf(stderr, "Proccess %d: " __FILE__ ": %d: fakenet server exiting.\n", getpid(), __LINE__);
			gm_net_shutdown();
			exit(0);
		} else {
			gint crv = -1;
			while (crv == -1)
				crv = gm_client_connect("127.0.0.1");
			fprintf(stderr, "Proccess %d: " __FILE__ ": %d: client connected to fakenet with return of %d.\n", getpid(), __LINE__, crv);
			usleep(2);
			monlist->monselect = &gm_trainer_net_select;
			monlist->itemselect = &gm_trainer_item_net_select;
			gm_battle_start(gm_trainer, &gm_battle_player_cb, monlist, &gm_battle_net_cb, FALSE, GM_MULTIPLAYER_CLIENT, TRUE);
			gm_net_shutdown();
		}
#else
		gm_battle_start(gm_trainer, &gm_battle_player_cb, monlist, &gm_battle_ai_cb, FALSE, GM_SINGLE_PLAYER, TRUE);
#endif
		rnd = monlist->nom;
		for (a = 0; a < rnd; a++)
			gm_trainer_delete_monster(monlist, monlist->monsters[0]->id);
		free(monlist);
	}
}

static void
gm_battle_nethost_select_cb(GtkWidget * widget, GtkWidget * window)
{
	gtk_widget_hide(window);
	gm_battle_nethost_cb();
	gtk_widget_show_all(window);
}

static void
gm_battle_nethost_cb()
{
	GMTrainer *monlist;
	gm_server_start();
	fprintf(stderr, "Proccess %d: " __FILE__ ": %d: network server initialized.\n", getpid(), __LINE__);
	usleep(2);
	gm_rand_init(GM_MULTIPLAYER_HOST);
	gm_trainer_send_net(gm_trainer);
//	gm_net_wait_write();
//	gm_net_wait();
	monlist = gm_trainer_load_net();
	monlist->monselect = &gm_trainer_net_select;
	monlist->itemselect = &gm_trainer_item_net_select;
	gm_battle_start(monlist, &gm_battle_net_cb, gm_trainer, &gm_battle_player_cb, FALSE, GM_MULTIPLAYER_HOST, TRUE);
	fprintf(stderr, "Proccess %d: " __FILE__ ": %d: network server exiting.\n", getpid(), __LINE__);
	gm_net_shutdown();
}

static void
gm_battle_network_select_cb(GtkWidget * widget, GtkWidget * window)
{
	GtkWidget *dialog;
	gchar ip[16];
	gtk_widget_hide(window);
	dialog = gnome_request_dialog(FALSE, "IP Address:", "", 15, gm_battle_network_ip, ip, NULL);
	gnome_dialog_run(GNOME_DIALOG(dialog));
	gm_battle_network_cb(ip);
	gtk_widget_show_all(window);
}

static void
gm_battle_network_cb(gchar *ip)
{
	GMTrainer *monlist;
	gint crv = -1;
	while (crv == -1)
		crv = gm_client_connect(ip);
	fprintf(stderr, "Proccess %d: " __FILE__ ": %d: client connected to network with return of %d.\n", getpid(), __LINE__, crv);
	usleep(2);
	gm_rand_init(GM_MULTIPLAYER_CLIENT);
//	gm_net_wait();
	monlist = gm_trainer_load_net();
	gm_trainer_send_net(gm_trainer);
//	gm_net_wait_write();
	monlist->monselect = &gm_trainer_net_select;
	monlist->itemselect = &gm_trainer_item_net_select;
	gm_battle_start(gm_trainer, &gm_battle_player_cb, monlist, &gm_battle_net_cb, FALSE, GM_MULTIPLAYER_CLIENT, TRUE);
	gm_net_shutdown();
}

static void
gm_battle_network_ip(gchar * ipaddr, gpointer data)
{
	strcpy(data, ipaddr);
}
