
#include "gmonsters.h"
#include "filedirs.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

void gm_at_exit();

gint gm_server_socket = 0;
gint gm_client_socket = 0;
GMTrainer *gm_trainer = NULL;
GMBattleType gm_rand_type;
FILE *lockfp = 0;

void
gm_init()
{
	FILE *fp = 0;
	gchar *lock = gnome_util_home_file("GMLock");
	gchar *tmp = gm_pixmap_file("gm-logo.png");
	guint trylock = 0;
	GtkWidget *initwin = gnome_dialog_new("GMonsters Startup", NULL);
	GtkWidget *logo = gnome_pixmap_new_from_file(tmp);
	GtkWidget *label = gtk_label_new("Initializing.....");
	GtkWidget *progress = gtk_progress_bar_new();
	free(tmp);
	gnome_window_icon_set_default_from_file(gm_pixmap_file("gnome-gmonsters.png"));
	gnome_window_icon_set_from_default(GTK_WINDOW(initwin));
	gtk_progress_configure(GTK_PROGRESS(progress), 0, 0, 100);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(initwin)->vbox), logo, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(initwin)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show_all(initwin);
	while (gtk_events_pending())
		gtk_main_iteration();
	lockfp = fopen(lock, "w+");
	fprintf(stderr, "Address of lock file is %x\n", lockfp);
	if (!lockfp || (trylock = lockf(fileno(lockfp), F_TEST, 0))) {
		fprintf(stderr, "Lock test returned %u\n", trylock);
		fprintf(stderr, "Couldn't use lock file [%s]!\nIs another GMonsters running?\nIf not just type 'rm %s' at the prompt.\nAborting...\n", lock, lock);
		exit(1);
	} else {
		fprintf(stderr, "Lock test returned %u\n", trylock);
		lockf(fileno(lockfp), F_LOCK, 0);
	}
	gm_species_load(GTK_PROGRESS_BAR(progress));
	while (gtk_events_pending())
		gtk_main_iteration();
	tmp = gnome_util_home_file("gmtrainer");
	fp = fopen(tmp, "r");
	if (fp) {
		gm_trainer = gm_trainer_load(fp);
		fclose(fp);
	} else
		gm_trainer = gm_trainer_new();
	gm_trainer->monselect = &gm_trainer_player_select;
	gm_trainer->itemselect = &gm_trainer_item_player_select;
	atexit(&gm_at_exit);
	gtk_widget_destroy(initwin);
	while (gtk_events_pending())
		gtk_main_iteration();
	free(tmp);
	free(lock);
}

void
gm_at_exit()
{
	gchar *tmp = gnome_util_home_file("gmtrainer");
	FILE *fp = fopen(tmp, "w");
	gchar *lock = gnome_util_home_file("GMLock");
	lockf(fileno(lockfp), F_ULOCK, 0);
	fclose(lockfp);
	remove(lock);
	gm_trainer_save(fp, gm_trainer);
	fclose(fp);
	gnome_config_sync();
	gnome_sound_shutdown();
	free(tmp);
	free(lock);
}

void
gm_server_start()
{
	gint client_len;
	struct sockaddr_in saddr;
	struct sockaddr_in caddr;
	gm_server_socket = socket(AF_INET, SOCK_STREAM, 0);
	saddr.sin_family = AF_INET;
	saddr.sin_addr.s_addr = htonl(INADDR_ANY);
	saddr.sin_port = htons(8139);
	bind(gm_server_socket, (struct sockaddr *)&saddr, sizeof(saddr));
	listen(gm_server_socket, 5);
	client_len = sizeof(caddr);
	gm_client_socket = accept(gm_server_socket, (struct sockaddr *)&caddr, &client_len);
}

gint
gm_client_connect(gchar * addr)
{
	struct sockaddr_in saddr;
	gm_server_socket = socket(AF_INET, SOCK_STREAM, 0);
	saddr.sin_family = AF_INET;
	saddr.sin_addr.s_addr = inet_addr(addr);
	saddr.sin_port = htons(8139);
	return connect(gm_server_socket, (struct sockaddr *)&saddr, sizeof(saddr));
}

void
gm_net_shutdown()
{
	if (gm_server_socket != 0)
		close(gm_server_socket);
	if (gm_client_socket != 0)
		close(gm_client_socket);
	gm_server_socket = 0;
	gm_client_socket = 0;
}

void
gm_rand_init(GMBattleType type)
{
	if (type == GM_SINGLE_PLAYER)
		srand((int)time(NULL));
	else if (type == GM_MULTIPLAYER_HOST)
		srand((int)time(NULL));
	gm_rand_type = type;
}

guint
gm_rand(guint high)
{
	guint rndnum = rand() % high;
	if (gm_rand_type == GM_MULTIPLAYER_HOST) {
		write(gm_client_socket, &rndnum, sizeof(guint));
		gm_net_wait_write();
	} else if (gm_rand_type == GM_MULTIPLAYER_CLIENT)
		read(gm_server_socket, &rndnum, sizeof(guint));
	return rndnum;
}

gint
gm_net_opponent()
{
	if (gm_rand_type == GM_MULTIPLAYER_HOST)
		return gm_client_socket;
	else if (gm_rand_type == GM_MULTIPLAYER_CLIENT)
		return gm_server_socket;
	return 0;
}

void
gm_net_wait()
{
	fd_set fdset;
	FD_ZERO(&fdset);
	FD_SET((gm_rand_type == GM_MULTIPLAYER_HOST) ? gm_client_socket : gm_server_socket, &fdset);
	select(FD_SETSIZE, &fdset, NULL, NULL, NULL);
}

void
gm_net_wait_write()
{
	fd_set fdset;
	FD_ZERO(&fdset);
	FD_SET((gm_rand_type == GM_MULTIPLAYER_HOST) ? gm_client_socket : gm_server_socket, &fdset);
	select(FD_SETSIZE, NULL, &fdset, NULL, NULL);
}
