
#include "gmonsters.h"
#include <sys/stat.h>

void gm_monster_level_up(GMMonster *, gboolean);

GMMonster **gm_monsters = NULL;
guint gm_monster_num = 0;

void
gm_monster_add(GMMonster * mon)
{
	GMMonster **tmp = gm_monsters;
	guint a;
	gm_monster_num++;
	gm_monsters = malloc(sizeof(GMMonster *) * gm_monster_num);
	gm_monsters[gm_monster_num - 1] = mon;
	if (tmp != NULL) {
		for (a = 0; a < gm_monster_num - 1; a++)
			gm_monsters[a] = tmp[a];
		free(tmp);
	}
}

void
gm_monster_remove(guint num)
{
	GMMonster **tmp = gm_monsters;
	guint a;
	if (num < gm_monster_num) {
		gm_monster_num--;
		if (gm_monster_num > 0) {
			gm_monsters = malloc(sizeof(GMMonster *) * gm_monster_num);
			for (a = 0; a < num; a++)
				gm_monsters[a] = tmp[a];
			for (a = num + 1; a < gm_monster_num + 1; a++)
				gm_monsters[a - 1] = tmp[a];
		} else
			gm_monsters = NULL;
		free(tmp);
	}
}

GMMonster *
gm_monster_new(GMSpecies * s, gchar * n, GMGender g, guint l)
{
	GMMonster *monster = malloc(sizeof(GMMonster));
	guint a, b, numact = 0;
	gchar *file = malloc(sizeof(gchar));
	do {
		free(file);
		monster->id = ((guint) rand()) + (((guint) rand()) >> 16);
		file = g_strdup_printf("gmonsters/%lu", monster->id);
	}
	while (access(gnome_util_home_file(file), F_OK) == TRUE);
	free(file);
	monster->spec = s;
	monster->name = strdup(n);
	monster->gen = g;
	monster->lvl = 1;
	monster->exp = 0;
	monster->maxhp = rand() % 3;
	monster->hp = gm_monster_maxhp(monster, 0);
	monster->str = rand() % 3;
	monster->def = rand() % 3;
	monster->spd = rand() % 3;
	monster->wis = rand() % 3;
	monster->nxp = gm_monster_maxhp(monster, 0) + gm_monster_str(monster, 0) + gm_monster_def(monster, 0) + gm_monster_spd(monster, 0) + gm_monster_int(monster, 0);
	monster->tamer = g_strdup("Wild (Not tamed)");
	monster->obe = 2 + (rand() % 9);
	monster->noa = 0;
	monster->abilities = 0;
	for (a = 0; a < monster->spec->alistlen; a++) {
		if (monster->spec->alist[a]->level == monster->lvl) {
			GMMonAbl **tmp1 = monster->abilities;
			monster->noa++;
			monster->abilities = malloc(sizeof(GMMonAbl *) * monster->noa);
			if (tmp1 != 0)
				for (b = 0; b < monster->noa - 1; b++)
					monster->abilities[b] = tmp1[b];
			monster->abilities[monster->noa - 1] = malloc(sizeof(GMMonAbl));
			monster->abilities[monster->noa - 1]->name = monster->spec->alist[a]->ability;
			monster->abilities[monster->noa - 1]->active = FALSE;
			for (b = 0; b < monster->noa; b++)
				if (monster->abilities[b]->active == TRUE)
					numact++;
			if (numact < 5)
				monster->abilities[monster->noa - 1]->active = TRUE;
			if (tmp1 != 0)
				free(tmp1);
		}
	}
	monster->ap = 100;
	while (monster->lvl < l)
		gm_monster_level_up(monster, FALSE);
	gm_monster_add(monster);
	return monster;
}

GMMonster *
gm_monster_open(gulong id)
{
	GMMonster *monster = NULL;
	FILE *fp;
	guint a;
	for (a = 0; a < gm_monster_num; a++)
		if (gm_monsters[a]->id == id)
			monster = gm_monsters[a];
	if (monster == NULL) {
		gchar *file = g_strdup_printf("gmonsters/%lu", id);
		if (access(gnome_util_home_file(file), F_OK) != -1) {
			gchar *tmp;
			guint len;
			monster = malloc(sizeof(GMMonster));
			fp = fopen(gnome_util_home_file(file), "r");
			fread(&len, sizeof(guint), 1, fp);
			monster->name = malloc(sizeof(gchar) * len);
			fread(monster->name, sizeof(gchar), len, fp);
			fread(&len, sizeof(guint), 1, fp);
			tmp = malloc(sizeof(gchar) * len);
			fread(tmp, sizeof(gchar), len, fp);
			monster->spec = gm_species_match(tmp);
			free(tmp);
			fread(&monster->gen, sizeof(GMGender), 1, fp);
			fread(&monster->lvl, sizeof(guint), 1, fp);
			fread(&monster->exp, sizeof(guint), 1, fp);
			fread(&monster->nxp, sizeof(guint), 1, fp);
			fread(&monster->maxhp, sizeof(guint), 1, fp);
			fread(&monster->hp, sizeof(guint), 1, fp);
			fread(&monster->str, sizeof(guint), 1, fp);
			fread(&monster->def, sizeof(guint), 1, fp);
			fread(&monster->spd, sizeof(guint), 1, fp);
			fread(&monster->wis, sizeof(guint), 1, fp);
			fread(&len, sizeof(guint), 1, fp);
			monster->tamer = malloc(sizeof(gchar) * len);
			fread(monster->tamer, sizeof(gchar), len, fp);
			fread(&monster->obe, sizeof(guint), 1, fp);
			fread(&monster->noa, sizeof(guint), 1, fp);
			monster->abilities = malloc(sizeof(GMMonAbl *) * monster->noa);
			fread(&monster->ap, sizeof(guint), 1, fp);
			monster->id = id;
			for (a = 0; a < monster->noa; a++) {
				fread(&len, sizeof(guint), 1, fp);
				tmp = malloc(sizeof(gchar) * len);
				fread(tmp, sizeof(gchar), len, fp);
				if (gm_ability_match(tmp)) {
					monster->abilities[a] = malloc(sizeof(GMMonAbl));
					monster->abilities[a]->name = tmp;
					fread(&monster->abilities[a]->active, sizeof(gboolean), 1, fp);
				} else
					monster->abilities[a] = 0;
			}
			gm_monster_add(monster);
			fclose(fp);
		}
		free(file);
	}
	return monster;
}

GMMonster *
gm_monster_open_net(gulong id)
{
	GMMonster *monster = NULL;
	guint a;
	gchar *tmp;
	guint len;
	for (a = 0; a < gm_monster_num; a++)
		if (gm_monsters[a]->id == id)
			gm_monster_remove(id);
	fprintf(stderr, __FILE__ ": %d: Reading monster from opponent...", __LINE__);
	monster = malloc(sizeof(GMMonster));
	gm_net_wait();
	read(gm_net_opponent(), &len, sizeof(guint));
	monster->name = malloc(sizeof(gchar) * len);
	gm_net_wait();
	read(gm_net_opponent(), monster->name, sizeof(gchar) * len);
	gm_net_wait();
	read(gm_net_opponent(), &len, sizeof(guint));
	tmp = malloc(sizeof(gchar) * len);
	gm_net_wait();
	read(gm_net_opponent(), tmp, sizeof(gchar) * len);
	monster->spec = gm_species_match(tmp);
	free(tmp);
	gm_net_wait();
	read(gm_net_opponent(), &monster->gen, sizeof(GMGender));
	gm_net_wait();
	read(gm_net_opponent(), &monster->lvl, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->exp, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->nxp, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->maxhp, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->hp, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->str, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->def, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->spd, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->wis, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &len, sizeof(guint));
	monster->tamer = malloc(sizeof(gchar) * len);
	gm_net_wait();
	read(gm_net_opponent(), monster->tamer, sizeof(gchar) * len);
	gm_net_wait();
	read(gm_net_opponent(), &monster->obe, sizeof(guint));
	gm_net_wait();
	read(gm_net_opponent(), &monster->noa, sizeof(guint));
	monster->abilities = malloc(sizeof(GMMonAbl *) * monster->noa);
	gm_net_wait();
	read(gm_net_opponent(), &monster->ap, sizeof(guint));
	monster->id = id;
	for (a = 0; a < monster->noa; a++) {
		gm_net_wait();
		read(gm_net_opponent(), &len, sizeof(guint));
		tmp = malloc(sizeof(gchar) * len);
		gm_net_wait();
		read(gm_net_opponent(), tmp, sizeof(gchar) * len);
		if (gm_ability_match(tmp)) {
			monster->abilities[a] = malloc(sizeof(GMMonAbl));
			monster->abilities[a]->name = tmp;
			gm_net_wait();
			read(gm_net_opponent(), &monster->abilities[a]->active, sizeof(gboolean));
		} else
			monster->abilities[a] = 0;
	}
	gm_monster_add(monster);
	return monster;
}

void
gm_monster_send_net(gulong id)
{
	GMMonster *monster = NULL;
	guint a;
	guint len;
//	for (a = 0; a < gm_monster_num; a++)
//		if (gm_monsters[a]->id == id)
//			monster = gm_monsters[a];
	monster = gm_monster_open(id);
	fprintf(stderr, __FILE__ ": %d: Sending monster to opponent...", __LINE__);
	len = strlen(monster->name) + 1;
	write(gm_net_opponent(), &len, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), monster->name, sizeof(gchar) * len);
	gm_net_wait_write();
	len = strlen(monster->spec->name) + 1;
	write(gm_net_opponent(), &len, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), monster->spec->name, sizeof(gchar) * len);
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->gen, sizeof(GMGender));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->lvl, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->exp, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->nxp, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->maxhp, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->hp, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->str, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->def, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->spd, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->wis, sizeof(guint));
	gm_net_wait_write();
	len = strlen(monster->tamer) + 1;
	write(gm_net_opponent(), &len, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), monster->tamer, sizeof(gchar) * len);
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->obe, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->noa, sizeof(guint));
	gm_net_wait_write();
	write(gm_net_opponent(), &monster->ap, sizeof(guint));
	gm_net_wait_write();
	for (a = 0; a < monster->noa; a++) {
		len = strlen(monster->abilities[a]->name) + 1;
		write(gm_net_opponent(), &len, sizeof(guint));
		gm_net_wait_write();
		write(gm_net_opponent(), monster->abilities[a]->name, sizeof(gchar) * len);
		gm_net_wait_write();
		write(gm_net_opponent(), &monster->abilities[a]->active, sizeof(gboolean));
		gm_net_wait_write();
	}
}

void
gm_monster_close(gulong id)
{
	FILE *fp;
	guint a, num = 0;
	for (a = 0; a < gm_monster_num; a++)
		if (gm_monsters[a]->id == id)
			num = a + 1;
	if (num != 0) {
		guint len = strlen(gm_monsters[num - 1]->name) + 1;
		gchar *file = g_strdup_printf("gmonsters/%lu", id);
		if (access(gnome_util_home_file("gmonsters"), F_OK) == -1)
			mkdir(gnome_util_home_file("gmonsters"), S_IRUSR | S_IWUSR | S_IXUSR);
		fp = fopen(gnome_util_home_file(file), "w");
		fwrite(&len, sizeof(guint), 1, fp);
		fwrite(gm_monsters[num - 1]->name, sizeof(gchar), len, fp);
		len = strlen(gm_monsters[num - 1]->spec->name) + 1;
		fwrite(&len, sizeof(guint), 1, fp);
		fwrite(gm_monsters[num - 1]->spec->name, sizeof(gchar), len, fp);
		fwrite(&gm_monsters[num - 1]->gen, sizeof(GMGender), 1, fp);
		fwrite(&gm_monsters[num - 1]->lvl, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->exp, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->nxp, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->maxhp, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->hp, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->str, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->def, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->spd, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->wis, sizeof(guint), 1, fp);
		len = strlen(gm_monsters[num - 1]->tamer) + 1;
		fwrite(&len, sizeof(guint), 1, fp);
		fwrite(gm_monsters[num - 1]->tamer, sizeof(gchar), len, fp);
		fwrite(&gm_monsters[num - 1]->obe, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->noa, sizeof(guint), 1, fp);
		fwrite(&gm_monsters[num - 1]->ap, sizeof(guint), 1, fp);
		for (a = 0; a < gm_monsters[num - 1]->noa; a++) {
			len = strlen(gm_monsters[num - 1]->abilities[a]->name) + 1;
			fwrite(&len, sizeof(guint), 1, fp);
			fwrite(gm_monsters[num - 1]->abilities[a]->name, sizeof(gchar), len, fp);
			fwrite(&gm_monsters[num - 1]->abilities[a]->active, sizeof(gboolean), 1, fp);
		}
		gm_monster_remove(num - 1);
		fclose(fp);
		free(file);
	}
}

void
gm_monster_destroy(gulong id)
{
	gchar *file = g_strdup_printf("gmonsters/%lu", id);
	guint a;
	for (a = 0; a < gm_monster_num; a++)
		if (gm_monsters[a]->id == id)
			gm_monster_remove(a);
	if (access(gnome_util_home_file(file), F_OK) != -1)
		remove(gnome_util_home_file(file));
	free(file);
}

void
gm_monster_learn_ability(GMMonster * mon, gchar * ability, gboolean popup)
{
	guint a, numact = 0;
	gboolean known = FALSE;
	for (a = 0; a < mon->noa; a++)
		if (!strcmp(ability, mon->abilities[a]->name))
			known = TRUE;
	if (known == FALSE) {
		GMMonAbl **tmp1 = mon->abilities;
		mon->noa++;
		mon->abilities = malloc(sizeof(GMMonAbl *) * mon->noa);
		for (a = 0; a < mon->noa - 1; a++) {
			mon->abilities[a] = tmp1[a];
		}
		mon->abilities[mon->noa - 1] = malloc(sizeof(GMMonAbl));
		mon->abilities[mon->noa - 1]->name = g_strdup(ability);
		mon->abilities[mon->noa - 1]->active = FALSE;
		for (a = 0; a < mon->noa; a++)
			if (mon->abilities[a]->active == TRUE)
				numact++;
		if (numact < 5)
			mon->abilities[mon->noa - 1]->active = TRUE;
		free(tmp1);
		if (popup == TRUE) {
			gchar *tmp = g_strdup_printf("%s learned %s!", mon->name, ability);
			gnome_dialog_run(GNOME_DIALOG(gnome_ok_dialog(tmp)));
			free(tmp);
		}
	}
}

void
gm_monster_give_exp(GMMonster * mon, guint exp, gboolean popup)
{
	if (mon->nxp > 0) {
		mon->exp += exp;
		if (popup == TRUE) {
			gchar *tmp = g_strdup_printf("%s gained %u experience!", mon->name, exp);
			gnome_dialog_run(GNOME_DIALOG(gnome_ok_dialog(tmp)));
			free(tmp);
		}
		while (mon->exp >= mon->nxp && mon->nxp != 0) {
			gm_monster_level_up(mon, popup);
		}
	}
}

void
gm_monster_level_up(GMMonster * mon, gboolean popup)
{
	guint a;
	if (mon->lvl < 99) {
		guint pmhp = gm_monster_maxhp(mon, 0);
		mon->lvl += 1;
		if (popup == TRUE) {
			gchar *tmp = g_strdup_printf("%s gained a level!\n%s is now at level %u!", mon->name,
										 (mon->gen == GM_MALE ? "He" : "She"), mon->lvl);
			gnome_dialog_run(GNOME_DIALOG(gnome_ok_dialog(tmp)));
			free(tmp);
		}
		if (mon->exp > mon->nxp)
			mon->exp -= mon->nxp;
		mon->hp = gm_monster_maxhp(mon, 0) - (pmhp - mon->hp);
		mon->nxp = gm_monster_maxhp(mon, 0) + gm_monster_str(mon, 0) + gm_monster_def(mon, 0) + gm_monster_spd(mon, 0) + gm_monster_int(mon, 0);
		if (mon->lvl > 98) {
			mon->lvl = 99;
			mon->exp = 0;
			mon->nxp = 0;
		}
		for (a = 0; a < mon->spec->alistlen; a++)
			if (mon->spec->alist[a]->level == mon->lvl)
				gm_monster_learn_ability(mon, mon->spec->alist[a]->ability, popup);
	}
}

guint
gm_monster_maxhp(GMMonster * mon, gint plus)
{
	return 10 + plus + ((mon->spec->shp + mon->maxhp) * mon->lvl);
}

guint
gm_monster_str(GMMonster * mon, gint plus)
{
	return 5 + plus + (((mon->spec->sstr + mon->str) * mon->lvl) / 5);
}

guint
gm_monster_def(GMMonster * mon, gint plus)
{
	return 5 + plus + (((mon->spec->sdef + mon->def) * mon->lvl) / 5);
}

guint
gm_monster_spd(GMMonster * mon, gint plus)
{
	return 5 + plus + (((mon->spec->sspd + mon->spd) * mon->lvl) / 5);
}

guint
gm_monster_int(GMMonster * mon, gint plus)
{
	return 5 + plus + (((mon->spec->swis + mon->wis) * mon->lvl) / 5);
}
