
#include "filedirs.h"
#include "species.h"

#if G_BYTE_ORDER == G_BE_ENDIAN
#define TWIST(val)	(val = GUINT32_SWAP_LE_BE_CONSTANT(val))
#else
#define TWIST(val) (val)
#endif

guint gm_species_num = 0;
GMSpecies *gm_species = 0;

void
gm_species_load(GtkProgressBar * progress)
{
	gchar *speciesfile = gm_data_file("gmspecies");
	FILE *fp = fopen(speciesfile, "r");
	guint a, b;
	if (fp) {
		fread(&gm_species_num, sizeof(guint), 1, fp);
#if G_BYTE_ORDER == G_BIG_ENDIAN
		gm_species_num = GUINT32_SWAP_LE_BE_CONSTANT(gm_species_num);
#endif
		gm_species = malloc(sizeof(GMSpecies) * gm_species_num);
		for (a = 0; a < gm_species_num; a++) {
			gchar *strng;
			gchar *tmp;
			guint len;
			gtk_progress_set_value(GTK_PROGRESS(progress), (a * 100) / gm_species_num);
			while (gtk_events_pending())
				gtk_main_iteration();
			fread(&len, sizeof(guint), 1, fp);
#if G_BYTE_ORDER == G_BIG_ENDIAN
			len = GUINT32_SWAP_LE_BE_CONSTANT(len);
#endif
			gm_species[a].name = malloc(sizeof(gchar) * len);
			fread(gm_species[a].name, sizeof(gchar), len, fp);
			strng = g_strdup_printf("%s.xpm", gm_species[a].name);
			tmp = gm_pixmap_file(strng);
			gm_species[a].image = gdk_imlib_load_image(tmp);
			free(tmp);
			free(strng);
			fread(&len, sizeof(guint), 1, fp);
#if G_BYTE_ORDER == G_BIG_ENDIAN
			len = GUINT32_SWAP_LE_BE_CONSTANT(len);
#endif
			gm_species[a].desc = malloc(sizeof(gchar) * len);
			fread(gm_species[a].desc, sizeof(gchar), len, fp);
			fread(&gm_species[a].type, sizeof(GMClassification), 1, fp);
			fread(&gm_species[a].shp, sizeof(guint), 1, fp);
			fread(&gm_species[a].sstr, sizeof(guint), 1, fp);
			fread(&gm_species[a].sdef, sizeof(guint), 1, fp);
			fread(&gm_species[a].sspd, sizeof(guint), 1, fp);
			fread(&gm_species[a].swis, sizeof(guint), 1, fp);
			fread(&gm_species[a].areas, sizeof(GMAreas), 1, fp);
			fread(&gm_species[a].alistlen, sizeof(guint), 1, fp);
#if G_BYTE_ORDER == G_BIG_ENDIAN
			gm_species[a].type = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].type);
			gm_species[a].shp = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].shp);
			gm_species[a].sstr = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].sstr);
			gm_species[a].sdef = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].sdef);
			gm_species[a].sspd = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].sspd);
			gm_species[a].swis = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].swis);
			gm_species[a].areas = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].areas);
			gm_species[a].alistlen = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].alistlen);
#endif
			gm_species[a].alist = malloc(sizeof(GMLearnAbility *) * gm_species[a].alistlen);
			for (b = 0; b < gm_species[a].alistlen; b++) {
				gm_species[a].alist[b] = malloc(sizeof(GMLearnAbility));
				fread(&gm_species[a].alist[b]->level, sizeof(guint), 1, fp);
				fread(&len, sizeof(guint), 1, fp);
#if G_BYTE_ORDER == G_BIG_ENDIAN
		gm_species[a].alist[b]->level = GUINT32_SWAP_LE_BE_CONSTANT(gm_species[a].alist[b]->level);
		len = GUINT32_SWAP_LE_BE_CONSTANT(len);
#endif
				gm_species[a].alist[b]->ability = malloc(sizeof(gchar) * len);
				fread(gm_species[a].alist[b]->ability, sizeof(gchar), len, fp);
			}
		}
	} else {
		fprintf(stderr, "Could not load species database.");
		exit(1);
	}
	gtk_progress_set_value(GTK_PROGRESS(progress), 100);
	free(speciesfile);
}

GMSpecies *
gm_species_get_list()
{
	return gm_species;
}

guint
gm_species_list_length()
{
	return gm_species_num;
}

GMSpecies *
gm_species_match(gchar * name)
{
	guint a = 0;
	for (a = 0; a < gm_species_num; a++) {
		if (!strcmp(gm_species[a].name, name))
			return gm_species + a;
	}
	return 0;
}
